# Meili Baragatti (Montpellier) and Emilie Lebarbier (Paris), 27/06/2017

####### Functions in this script
# line 16 : simuSeries:  Simulation of series
# line 133 : segmentation of series corrupted by a functional part
#      line 137 : segmentation_bias_MH : estimation of the breakpoints and of the functions composing the functional part (Metropolis-Hastings algorithm)
#      line 302 : estimation_moy_biais: After the segmentation, estimation of betagamma, lambdar and sigma2 (Gibbs sampler)
# line 416 : segmentation of series without taking into account the corruption by a functional part
#      line 420 : segmentation_nobias_MH : estimation of the breakpoints (Metropolis-Hastings algorithm)
#      line 508 : estimation_moy_nobiais : After the segmentation, estimation of betagamma and sigma2 (Gibbs sampler)

library(Matrix)
library(LearnBayes)
library(MCMCpack)

#####################################################################################
#                               Simulation of series                                #
#####################################################################################

# function to simulate M series, each one having their own breakpoints, but all sharing the same number of segments, and the same functional part which is specified in the article.

# INPUTS
# M the number of series (if M>1, we are in case of multplie series)
# n the number of points of the series
# K: the number of segments for each serie
# mu.choix vector which gives the possible values for each segment of the segmentation part of the series. The values of each segment are unifomly sampled from this vector, and the value of a segment should not be equal to the value of the preceding segment.
# varianceError: vector of variances for the error term
# pHaar: the positions of the Haar functions composing the functional part
# p1: A breakpoint should be positioned at a distance from the Haar functions of at least p1
# p2: Each segment is at least of length p2

# OUTPUTS
# K: the number of segments for each serie
# muMat: matrix (M x n) which gives the value of the segmentation part of each serie at each position
# tauMat: matrix (M x k.max) which gives the index of the breakpoints for each series (K points for each series)
# errorMat: matrice (M x n) which gives the error term at every time point, for each series
# biais: evaluation of the functional part at every time point
# Profile: data.frame which summarizes all the preceding informations in columns: number (id) of the series, position in the grid of time points, value of the segmentation part at this position, presence or not of a breakpoint at this position, value of the functional part at this position, error term at this position (one column for each specified variance of the error term)

simuSeries <- function(M,n,K,mu.choix,varianceError,pHaar,p1,p2)  
{
  # Construction of the functional part f
  t <- 1:n
  A <- 0.3
  Part1 <- A*sin(2*pi*t/20)
  Part2 <- rep(0,n)
  t1 <- pHaar[1]
  t2 <- pHaar[2]
  t3 <- pHaar[3]
  Part2[t1] <- 1.5
  Part2[t2] <- -2
  Part2[t3] <- 3
  biais <- Part1+Part2
  
  # construction of the M series
  erreurs <- list()
  muMat  <-  matrix(nrow=M, ncol=n)
  tauMat  <-  matrix(NaN, nrow=M, ncol=K)
  
  # errors: a matrix (M x n) for each possible value of variance, giving the errors of each series at each position
  for (i in 1:length(varianceError)){
    errorMat  <-  matrix(nrow=M, ncol=n)
    for (m in 1:M){
      errorMat[m, ] = rnorm(n,m=0,sd=sqrt(varianceError[i]))
    }
    erreurs[[i]] <- errorMat
  }
  
  for (m in (1:M))
  {
    # positions of breakpoints pour series m
    cond <- 0
    while (cond==0){
      tau.tmp = ceiling((n-1)*runif(K-1))
      tau.tmp = sort(tau.tmp)  
      tau.tmp <- c(tau.tmp, n)
      cond2 <- 1
      for (i in 2:K){
        cond2 <- cond2 * ((tau.tmp[i]-tau.tmp[i-1])>=p2)
      }
      for (i in 1:K){
        for (j in 1:length(pHaar)){
          cond2 <- cond2 * abs((tau.tmp[i]-pHaar[j]))>=p1
        }
      }
      cond <- cond2
    }   
    tauMat[m, (1:K)] = tau.tmp
    
    # values of segments (segmentation part) for series m
    mutemp <- sample(mu.choix,size=1)
    muMat[m,1:tauMat[m, 1]] <- rep(mutemp,tauMat[m, 1]) 
    
    if (K > 1){
      for (k in 2:K){
        toremove <- which(mu.choix==mutemp)
        mu.choix.temp <- mu.choix[-toremove]
        mutemp <- sample(mu.choix.temp,size=1)
        muMat[m,(tauMat[m, k-1]+1):tauMat[m, k]] <- rep(mutemp,(tauMat[m, k]-tauMat[m, k-1])) 
      } 
    }
  }
  
  # final output as a data.frame
  biais.rep  <-  rep(biais,M)
  series <- rep(c(1:M),each=n)
  position <-rep(c(1:n),M)
  mu <- matrix(t(muMat),nrow=M*n,ncol=1) 
  errors <- matrix(t(erreurs[[1]]),nrow=M*n,ncol=1)
  nomcolonnes <- "erreur1"
  if (length(varianceError)>1){
    for (i in 2:length(varianceError)){
      errors <- cbind(errors, matrix(t(erreurs[[i]]),nrow=M*n,ncol=1))
      nomcolonnes <- c(nomcolonnes,paste("erreur",i,sep=""))
    }    
  }
  tau <- numeric()
  for (m in (1:M))
  {
    tau.pos <- rep(0,n)
    tau.pos[tauMat[m, ]] <- 1
    tau <- c(tau, tau.pos)
  }
  Profile <- data.frame(series=series,position=position,mu=mu,tau=tau,biais=biais.rep,errors)
  data.frame(series=series,position=position,mu=mu,tau=tau,biais=biais.rep)
  colnames(Profile) <- c("serie","position","mu","tau","biais",nomcolonnes)
  # output
  return(list(K,muMat,tauMat,errorMat,biais,Profile))
}



############################################################################################
#                 segmentation of series corrupted by a functional part                    #
############################################################################################

############################
# estimation of the breakpoints and of the functions composing the functional part (Metropolis-Hastings algorithm)
############################

# INPUTS
# serie: the observations
# nbiter: number of MH iterations total (burn-in + post burn-in)
# nburn: number of burn-in iterations
# lec1: chosen value for parameter c1
# lec2: chosen value for parameter c2
# Fmatrix: matrix which gives the values of the functions in the dictionary, at each position.
# nbSegInit: initial number of segments for the segmentation part
# nbToChangegamma: number of gamma components proposed to be changed at each iteration (when we propose to modify gamma), that is the number of inclusion or deletion of breakpoints
# nbFuncInit: initial number of functions from the dictionary for the functional part
# nbToChanger: number of r components proposed to be changed at each iteration (when we propose to modify r), that is the number of inclusion or deletion of functions from the dictionary
# Pi: vector with prior probabilities for the breakpoints: for position l it is the prior probability taht we observe a difference between the segmentation part at time l and the segmentation part at time (l-1). By convention Pi[1] = 1.
# eta: vector with prior probabilities for the functions in the dictionary: eta[j] gives the prior proba that the function j from the dictionary will be included in the functional part. By convention eta[1] = 1.
# printiter: if TRUE, the number of the actual iteration is plotted

# OUTPUTS
# sumgamma: for the breakpoints, for component l: number of iterations during which a difference between the segmentation part at time l and the segmentation part at time (l-1) was non nul (during post-burn-in).
# sumr: for functions from the dictionary, for component l: number of iterations during which the function j from the dictionary was included in the functional part (during post-burn-in).
# nbactugamma: number of iterations during which a MH proposal for the breakpoints has been accepted (during which gamma has been modified) (among iterations during burn-in and post-burn-in).
# nbactur: number of iterations during which a MH proposal for the functions from the dictionary has been accepted (during which r has been modified) (among iterations during burn-in and post-burn-in).
# gammamatrix: matrix to store all the post-burn-in simulations for gamma: one line corresponds to one simulation (iteration)
# rmatrix: matrix to store all the post-burn-in simulations for gamma r: one line corresponds to one simulation (iteration)

segmentation_bias_MH <- function (serie, nbiter, nburn, lec1, lec2, Fmatrix, nbSegInit,nbToChangegamma,nbFuncInit,nbToChanger,Pi,eta,printiter=TRUE) 
{
  y <- as.numeric(as.vector(serie))
  # y <- y - mean(y)
  n <- length(y)
  X <- diag(n)
  X[lower.tri(X)] <- 1 
  J <- dim(Fmatrix)[2]
  
  # to store results
  gammamatrix <- matrix(ncol=n,nrow=(nbiter-nburn))
  rmatrix <- matrix(ncol=J,nrow=(nbiter-nburn))
  sumgamma <- rep(0, n)
  nbactugamma <- 0
  sumr <- rep(0, J)
  nbactur <- 0  
  # initialisation gamma
  indgamma10 <- sample(c(2:n), nbSegInit-1, replace = FALSE)
  gamma0 <- rep(0, n)
  gamma0[1] <- 1
  for (i in 1:(nbSegInit-1)) {
    gamma0[indgamma10[i]] <- 1
  }
  indgamma1 <- c(1,indgamma10)
  gamma <- gamma0
  nbSeg <- nbSegInit
  Xgamma <- X[, indgamma1]
  invUgamma <- diag(n) - lec1/(1+lec1)*Xgamma%*%solve(t(Xgamma)%*%Xgamma)%*%t(Xgamma)
  # initialisation r
  indr10 <- sample(c(2:J), nbFuncInit-1, replace = FALSE)
  r0 <- rep(0, J)
  r0[1] <- 1
  for (i in 1:(nbFuncInit-1)) {
    r0[indr10[i]] <- 1
  }
  indr1 <- c(1,indr10)
  r <- r0
  nbFunc <- nbFuncInit
  Fmatrixr <- Fmatrix[, indr1]
  

  temp1 <- det(solve(t(Fmatrixr)%*%(invUgamma+diag(n)/lec2)%*%Fmatrixr))
  temp2 <- (t(y)%*%(invUgamma-invUgamma%*%Fmatrixr%*%solve(t(Fmatrixr)%*%(invUgamma+diag(n)/lec2)%*%Fmatrixr)%*%t(Fmatrixr)%*%invUgamma)%*%y)
  temp3 <- det(t(Fmatrixr)%*%Fmatrixr)
  
  # iterations MH
  for (iter in 1:nbiter) {
    if (printiter==TRUE){
      print(paste("iter",iter,sep=" ") )
    }
    # uniform choice between several movements
    choix <- sample(c(1:2),size=1)
    # movement proposing to change only gamma
    if (choix == 1){
      gammaprop <- gamma
      indgamma1prop <- indgamma1
      nbSegprop <- nbSeg
      indToChange <- sample(c(2:n), nbToChangegamma, replace = FALSE)
      for (i in 1:nbToChangegamma){
        if (gamma[indToChange[i]]==0){
          gammaprop[indToChange[i]] <- 1
          indgamma1prop <- c(indgamma1prop,indToChange[i])
          nbSegprop <- nbSegprop + 1
        }
        else {
          gammaprop[indToChange[i]] <- 0
          indremove <- which(indgamma1prop==indToChange[i])
          indgamma1prop <- indgamma1prop[-indremove]
          nbSegprop <- nbSegprop - 1
        }
      }
      Xgammaprop <- X[, indgamma1prop]
      invUgammaprop <- diag(n) - lec1/(1+lec1)*Xgammaprop%*%solve(t(Xgammaprop)%*%Xgammaprop)%*%t(Xgammaprop)
      temp1prop <- det(solve(t(Fmatrixr)%*%(invUgammaprop+diag(n)/lec2)%*%Fmatrixr))
      temp2prop <- (t(y)%*%(invUgammaprop-invUgammaprop%*%Fmatrixr%*%solve(t(Fmatrixr)%*%(invUgammaprop+diag(n)/lec2)%*%Fmatrixr)%*%t(Fmatrixr)%*%invUgammaprop)%*%y)
      A <- (1+lec1)^((nbSeg-nbSegprop)/2) * prod((Pi[-1]/(1-Pi[-1]))^((gammaprop-gamma)[-1])) * (temp1prop/temp1)^(1/2) * (temp2/temp2prop)^(n/2)
      probaccept1 <- min(1,A)
      seuil <- runif(1)
      if (seuil < probaccept1){
        gamma <- gammaprop
        indgamma1 <- indgamma1prop
        nbSeg <- nbSegprop
        Xgamma <- Xgammaprop
        invUgamma <- invUgammaprop
        temp1 <- temp1prop
        temp2 <- temp2prop
        nbactugamma <- nbactugamma+1
      }                
    }
    # movement proposing to change only r
    if (choix == 2){
      rprop <- r
      indr1prop <- indr1
      nbFuncprop <- nbFunc
      indToChange <- sample(c(2:J), nbToChanger, replace = FALSE)
      for (i in 1:nbToChanger){
        if (r[indToChange[i]]==0){
          rprop[indToChange[i]] <- 1
          indr1prop <- c(indr1prop,indToChange[i])
          nbFuncprop <- nbFuncprop + 1
        }
        else {
          rprop[indToChange[i]] <- 0
          indremove <- which(indr1prop==indToChange[i])
          indr1prop <- indr1prop[-indremove]
          nbFuncprop <- nbFuncprop - 1
        }
      }
      Fmatrixrprop <- Fmatrix[, indr1prop]
      temp1prop <- det(solve(t(Fmatrixrprop)%*%(invUgamma+diag(n)/lec2)%*%Fmatrixrprop))
      temp2prop <- (t(y)%*%(invUgamma-invUgamma%*%Fmatrixrprop%*%solve(t(Fmatrixrprop)%*%(invUgamma+diag(n)/lec2)%*%Fmatrixrprop)%*%t(Fmatrixrprop)%*%invUgamma)%*%y)
      temp3prop <- det(t(Fmatrixrprop)%*%Fmatrixrprop)
      A <- lec2^((nbFunc-nbFuncprop)/2) * prod((eta[-1]/(1-eta[-1]))^((rprop-r)[-1])) * (temp1prop/temp1)^(1/2) * (temp2/temp2prop)^(n/2) * (temp3prop/temp3)^(1/2)
      probaccept1 <- min(1,A)
      seuil <- runif(1)
      if (seuil < probaccept1){
        r <- rprop
        indr1 <- indr1prop
        nbFunc <- nbFuncprop
        Fmatrixr <- Fmatrixrprop
        temp1 <- temp1prop
        temp2 <- temp2prop
        temp3 <- temp3prop
        nbactur <- nbactur+1
      }      
    }
    # store results when we are in post-burn-in
    if (iter > nburn) {
      sumgamma <- sumgamma + gamma
      sumr <- sumr + r
      gammamatrix[iter-nburn,] <- gamma
      rmatrix[iter-nburn,] <- r
    }
  }
  return(list(sumgamma,sumr,nbactugamma,nbactur,gammamatrix,rmatrix))
}


############################
#  After the segmentation: Estimation of betagamma, lambdar and sigma2 (Gibbs sampler)   
############################

# INPUTS
# serie: the observations
# nbiter: total number of iterations for the Gibbs sampler (burn-in + post burn-in)
# nburn: number of burn-in iterations
# lec1: chosen value for the c1 parameter
# lec2: chosen value for the c2 parameter
# Fmatrix: matrix which gives the values of the functions in the dictionary, at each position.
# gammahat: estimated gamma vector (using the preceding MH algo) 
# rhat: estimated r vector (using the preceding MH algo) 
# priorminsigma2: prior minimum value for sigma2
# priormaxsigma2: prior maximum value for sigma2
# printiter: if TRUE, the number of the actual iteration is plotted

# OUTPUTS
# resbetagammahat: matrix to store all the post-burn-in simulations for betagamma: one line corresponds to one simulation (iteration)
# reslambdarhat: matrix to store all the post-burn-in simulations for lambdar: one line corresponds to one simulation (iteration)
# ressigma2hat: matrix to store all the post-burn-in simulations for sigma2: one line corresponds to one simulation (iteration)
# estbetagamma: estimation of betagamma (mean of all the simulated betagamma)
# estlambdar: estimation of lambdar (mean of all the simulated lambdar)
# estsigma2: estimation of sigma2 (mean of all the simulated sigma2)

estimation_moy_biais <- function (serie, nbiter, nburn, lec1, lec2, Fmatrix, gammahat,rhat,priorminsigma2,priormaxsigma2,printiter=TRUE) 
{
   y <- as.numeric(as.vector(serie))
   # y <- y - mean(y)
   n <- length(y)
   X <- diag(n)
   X[lower.tri(X)] <- 1
   J <- dim(Fmatrix)[2]
   dgammahat <- sum(gammahat)
   drhat <- sum(rhat)
   
   if (drhat!=0){
     # useful quantities for calculations
     Xgamma <- as.matrix(X[,which(gammahat==1)])
     Fmatrixr <- as.matrix(Fmatrix[,which(rhat==1)])
     temp1 <- solve((1+lec1)/lec1*t(Xgamma)%*%Xgamma)
     temp2 <- solve((1+lec2)/lec2*t(Fmatrixr)%*%Fmatrixr)
     alpha <- (n+dgammahat+drhat)/2
     temp3 <- t(Xgamma)%*%Xgamma/lec1
     temp4 <- t(Fmatrixr)%*%Fmatrixr/lec2
     # to store results
     resbetagammahat <- matrix(nrow=dgammahat,rep(0, dgammahat*(nbiter-nburn)))
     reslambdarhat <- matrix(nrow=drhat,rep(0, drhat*(nbiter-nburn)))
     ressigma2hat <- rep(0, nbiter-nburn)
     # initialisations
     sigma2hat <- exp(runif(1,min=log(priorminsigma2),max=log(priormaxsigma2)))
     betagammahat <- as.numeric(rmnorm(n=1,mean=rep(0,dgammahat),varcov=lec1*sigma2hat*solve(t(Xgamma)%*%Xgamma)))
     lambdarhat <- as.numeric(rmnorm(n=1,mean=rep(0,drhat),varcov=lec2*sigma2hat*solve(t(Fmatrixr)%*%Fmatrixr)))
     # iterations
     for (iter in 1:nbiter) {
       if (printiter==TRUE){
         print(paste("iter",iter,sep=" ") )
       }
       ordre <- sample(1:3,size=3,replace=FALSE)
       for (j in 1:3){
         if (ordre[j]==1){ betagammahat <- as.numeric(rmnorm(n=1,mean=temp1%*%t(Xgamma)%*%(y-Fmatrixr%*%lambdarhat),varcov=sigma2hat*temp1)) }
         if (ordre[j]==2){ lambdarhat <- as.numeric(rmnorm(n=1,mean=temp2%*%t(Fmatrixr)%*%(y-Xgamma%*%betagammahat),varcov=sigma2hat*temp2)) }
         if (ordre[j]==3){ sigma2hat <- rinvgamma(n=1,shape=alpha,scale=1/2*(t(y-Xgamma%*%betagammahat-Fmatrixr%*%lambdarhat)%*%(y-Xgamma%*%betagammahat-Fmatrixr%*%lambdarhat)+t(betagammahat)%*%temp3%*%betagammahat+t(lambdarhat)%*%temp4%*%lambdarhat)) }        
       }
       
       if (iter >= (nburn+1)){
         resbetagammahat[,iter-nburn]  <- betagammahat
         reslambdarhat[,iter-nburn] <- lambdarhat
         ressigma2hat[iter-nburn] <- sigma2hat
       }
     }
     estbetagamma <- apply(resbetagammahat,1,sum)/(nbiter-nburn)
     estlambdar <- apply(reslambdarhat,1,sum)/(nbiter-nburn)
     estsigma2 <- sum(ressigma2hat)/(nbiter-nburn)
   }

   if (drhat==0){
     # useful quantities for calculations
     Xgamma <- as.matrix(X[,which(gammahat==1)])
     temp1 <- solve((1+lec1)/lec1*t(Xgamma)%*%Xgamma)
     alpha <- (n+dgammahat)/2
     temp3 <- t(Xgamma)%*%Xgamma/lec1
     # to store results
     resbetagammahat <- matrix(nrow=dgammahat,rep(0, dgammahat*(nbiter-nburn)))
     reslambdarhat <- NULL
     ressigma2hat <- rep(0, nbiter-nburn)
     # initialisations
     sigma2hat <- exp(runif(1,min=log(priorminsigma2),max=log(priormaxsigma2)))
     betagammahat <- as.numeric(rmnorm(n=1,mean=rep(0,dgammahat),varcov=lec1*sigma2hat*solve(t(Xgamma)%*%Xgamma)))
     # iterations
     for (iter in 1:nbiter) {
       if (printiter==TRUE){
         print(paste("iter",iter,sep=" ") )
       }
       ordre <- sample(1:2,size=2,replace=FALSE)
       for (j in 1:2){
         if (ordre[j]==1){ betagammahat <- as.numeric(rmnorm(n=1,mean=temp1%*%t(Xgamma)%*%y,varcov=sigma2hat*temp1)) }
         if (ordre[j]==2){ sigma2hat <- rinvgamma(n=1,shape=alpha,scale=1/2*(t(y-Xgamma%*%betagammahat)%*%(y-Xgamma%*%betagammahat)+t(betagammahat)%*%temp3%*%betagammahat)) }        
       }
       
       if (iter >= (nburn+1)){
         resbetagammahat[,iter-nburn]  <- betagammahat
         ressigma2hat[iter-nburn] <- sigma2hat
       }
     }  
     estbetagamma <- apply(resbetagammahat,1,sum)/(nbiter-nburn)
     estlambdar <- NULL
     estsigma2 <- sum(ressigma2hat)/(nbiter-nburn)
   }
   
   return(list(resbetagammahat,reslambdarhat,ressigma2hat,estbetagamma,estlambdar,estsigma2))
}


################################################################################################
#   segmentation of series without taking into account the corruption by a functional part     #
################################################################################################

############################
# estimation of the breakpoints (Metropolis-Hastings algorithm)
############################

# INPUTS
# serie: the observations
# nbiter: number of MH iterations total (burn-in + post burn-in)
# nburn: number of burn-in iterations
# lec: chosen value for parameter c
# nbSegInit: initial number of segments for the segmentation part
# nbToChange: number of gamma components proposed to be changed at each iteration (when we propose to modify gamma), that is the number of inclusion or deletion of breakpoints
# Pi: vector with prior probabilities for the breakpoints: for position l it is the prior probability taht we observe a difference between the segmentation part at time l and the segmentation part at time (l-1). By convention Pi[1] = 1.
# printiter: if TRUE, the number of the actual iteration is plotted

# OUTPUTS
# sumgamma: for the breakpoints, for component l: number of iterations during which a difference between the segmentation part at time l and the segmentation part at time (l-1) was non nul (during post-burn-in).
# nbactugamma: number of iterations during which a MH proposal for the breakpoints has been accepted (during which gamma has been modified) (among iterations during burn-in and post-burn-in).
# nbSegactu:  each time gamma is modified, we store the number of segments of the segmentation part corresponding to this new gamma value (included burn-in)

segmentation_nobias_MH <- function (serie, nbiter, nburn, lec,
                                    nbSegInit,nbToChange,lambda,printiter=TRUE) 
{
  y <- as.numeric(as.vector(serie))
  # y <- y - mean(y)
  n <- length(y)
  X <- diag(n)
  X[lower.tri(X)] <- 1 
  
  # initialisation
  sumgamma <- rep(0, n)
  nbactu <- 0
  nbSegactu <- numeric()
  indgamma10 <- sample(c(2:n), nbSegInit-1, replace = FALSE)
  gamma0 <- rep(0, n)
  gamma0[1] <- 1
  for (i in 1:(nbSegInit-1)) {
    gamma0[indgamma10[i]] <- 1
  }
  indgamma1 <- c(1,indgamma10)
  gamma <- gamma0
  nbSeg <- nbSegInit
  Xgamma <- X[, indgamma1]
  temp <- t(y)%*%(diag(rep(1,n))-lec/(1+lec)*Xgamma%*%solve(t(Xgamma)%*%Xgamma)%*%t(Xgamma))%*%y
  
  # iterations MH
  for (iter in 1:nbiter) {
    if (printiter==TRUE){
      print(paste("iter",iter,sep=" ") )
    }
    gammaprop <- gamma
    indgamma1prop <- indgamma1
    indToChange <- sample(c(2:n), nbToChange, replace = FALSE)
    for (i in 1:nbToChange){
      if (gamma[indToChange[i]]==0){
        gammaprop[indToChange[i]] <- 1
        indgamma1prop <- c(indgamma1prop,indToChange[i])
      }
      else {
        gammaprop[indToChange[i]] <- 0
        indremove <- which(indgamma1prop==indToChange[i])
        indgamma1prop <- indgamma1prop[-indremove]
      }
    }
    nbSegprop <- length(indgamma1prop)
    Xgammaprop <- X[, indgamma1prop]
    tempprop <- (t(y)%*%(diag(rep(1,n))-lec/(1+lec)*Xgammaprop
                         %*%solve(t(Xgammaprop)%*%Xgammaprop)%*%t(Xgammaprop))%*%y)
    A <- (1+lec)^((nbSeg-nbSegprop)/2) * prod((lambda[-1]/(1-lambda[-1]))^((gammaprop-gamma)[-1])) * (temp/tempprop)^(n/2)
    probaccept1 <- min(1,A)
    seuil <- runif(1)
    if (seuil < probaccept1){
      gamma <- gammaprop
      indgamma1 <- indgamma1prop
      Xgamma <- Xgammaprop
      nbSeg <- nbSegprop
      temp <- tempprop
      nbactu <- nbactu+1
      nbSegactu <- c(nbSegactu,nbSeg)
    }
    # Store results after brun-in
    if (iter > nburn) {
      sumgamma <- sumgamma + gamma
    }
  }
  return(list(sumgamma,nbactu,nbSegactu))
}


############################
# After the segmentation, estimation of betagamma and sigma2 (Gibbs sampler)
############################

# INPUTS
# serie: the observations
# nbiter: total number of iterations for the Gibbs sampler (burn-in + post burn-in)
# nburn: number of burn-in iterations
# lec1: chosen value for the c1 parameter
# gammahat: estimated gamma vector (using the preceding MH algo) 
# priorminsigma2: prior minimum value for sigma2
# priormaxsigma2: prior maximum value for sigma2
# printiter: if TRUE, the number of the actual iteration is plotted

# OUTPUTS
# resbetagammahat: matrix to store all the post-burn-in simulations for betagamma: one line corresponds to one simulation (iteration)
# sigma2hat: matrix to store all the post-burn-in simulations for sigma2: one line corresponds to one simulation (iteration)
# estbetagamma: estimation of betagamma (mean of all the simulated betagamma)
# estsigma2: estimation of sigma2 (mean of all the simulated sigma2)

estimation_moy_nobiais <- function (serie, nbiter, nburn, lec1, gammahat, priorminsigma2,priormaxsigma2,printiter=TRUE) 
{
  y <- as.numeric(as.vector(serie))
  # y <- y - mean(y)
  n <- length(y)
  X <- diag(n)
  X[lower.tri(X)] <- 1
  dgammahat <- sum(gammahat)
  
    # useful values for calculation
    Xgamma <- as.matrix(X[,which(gammahat==1)])
    temp1 <- solve((1+lec1)/lec1*t(Xgamma)%*%Xgamma)
    alpha <- (n+dgammahat)/2
    temp3 <- t(Xgamma)%*%Xgamma/lec1
    # to store results
    resbetagammahat <- matrix(nrow=dgammahat,rep(0, dgammahat*(nbiter-nburn)))
    ressigma2hat <- rep(0, nbiter-nburn)
    # initialisations
    sigma2hat <- exp(runif(1,min=log(priorminsigma2),max=log(priormaxsigma2)))
    betagammahat <- as.numeric(rmnorm(n=1,mean=rep(0,dgammahat),varcov=lec1*sigma2hat*solve(t(Xgamma)%*%Xgamma)))
    # iterations
    for (iter in 1:nbiter) {
      if (printiter==TRUE){
        print(paste("iter",iter,sep=" ") )
      }
      ordre <- sample(1:2,size=2,replace=FALSE)
      for (j in 1:2){
        if (ordre[j]==1){ betagammahat <- as.numeric(rmnorm(n=1,mean=temp1%*%t(Xgamma)%*%y,varcov=sigma2hat*temp1)) }
        if (ordre[j]==2){ sigma2hat <- rinvgamma(n=1,shape=alpha,scale=1/2*(t(y-Xgamma%*%betagammahat)%*%(y-Xgamma%*%betagammahat)+t(betagammahat)%*%temp3%*%betagammahat)) }        
      }
      
      if (iter >= (nburn+1)){
        resbetagammahat[,iter-nburn]  <- betagammahat
        ressigma2hat[iter-nburn] <- sigma2hat
      }
    }  
    estbetagamma <- apply(resbetagammahat,1,sum)/(nbiter-nburn)
    estsigma2 <- sum(ressigma2hat)/(nbiter-nburn)
  
  return(list(resbetagammahat,ressigma2hat,estbetagamma,estsigma2))
}



